//
//  HSArrayDivide.h
//  HSCore
//
//  Created by Kyle Hankinson on 2019-12-16.
//  Copyright © 2019 Hankinsoft Development, Inc. All rights reserved.
//

#ifndef HSArrayDivide_h
#define HSArrayDivide_h

#import "MachTimer.h"

static inline void HSArrayDivide(NSArray **slices, NSArray *inArray, NSUInteger parts) {
    MachTimer * divideTimer = [MachTimer startTimer];
    pspdf_defer {
        CGFloat elapsedTime = divideTimer.elapsedSeconds;
        NSLog(@"Autocomplete - DMArrayDivide took %0.2f seconds.", elapsedTime);
    };

    if (parts == 0)
    {
        *slices = NULL;
        return;
    }

    if(nil == inArray)
    {
        *slices = NULL;
        return;
    } // End of inArray

    // Get our items per array. If we cannot split it into enough parts, then
    // we will try and size it up properly.
    NSUInteger itemsPerArray = 0;
    while(0 == itemsPerArray && parts > 0)
    {
        CGFloat partial = (CGFloat)inArray.count / (CGFloat) parts;
        itemsPerArray = floor(partial);

        if(0 == itemsPerArray)
        {
            --parts;
            continue;
        }
    }

    NSMutableArray *arrays = [NSMutableArray arrayWithCapacity: parts];
    for (NSUInteger i = 0; i < parts; i++)
    {
        NSUInteger startLocation = i * itemsPerArray;
        NSRange range = NSMakeRange(startLocation, itemsPerArray);

        // If we have leftovers, add them to our last entries
        if(i == parts - 1)
        {
            range = NSMakeRange(startLocation, inArray.count - startLocation);
        }

        NSArray * tempArray = [inArray subarrayWithRange: range];
        arrays[i] = tempArray;
    }

    *slices = arrays;
}

#endif /* HSArrayDivide_h */
